import os
import glob
import click
import requests
import subprocess

from askanna.cli.utils import check_for_project

# FIXME: this will be an AskAnna public link in the future
# For dev purposes, add your local path to the cookiecutter template
# ASKANNA_FILEUPLOAD_ENDPOINT = "REPLACEME"
ASKANNA_FILEUPLOAD_ENDPOINT = "http://localhost:8000/u/api/upload/"

HELP = """
This command will allow you to deploy a DSP - Data Science Project to
the AskAnna platform.
"""

SHORT_HELP = "Deploy a Data Science Project to AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():

    pwd = os.getcwd()

    print("We are located in: {pwd}".format(pwd=pwd))

    if not check_for_project():
        click.echo("Not in a project folder, exiting")
        return 0

    click.echo("\nPackaging your project...")
    export_file = '/tmp/{}_package/'.format(pwd.split('/')[-1])
    # subprocess.Popen(f"python setup.py sdist bdist_wheel -d {export_file}",
    #                 shell=True)
    subprocess.run(
        ["python", "setup.py", "sdist", "bdist_wheel", "-d", export_file],
        capture_output=True)

    wheel_package = glob.glob(export_file + '*.whl')[0]
    click.echo("Finished package: {}".format(wheel_package.split('/')[-1]))
    click.echo("Uploading to AskAnna...")

    # path_to_wheel = '/tmp/barShop/dist/barShop-0.1.0-py3-none-any.whl'
    # files = {'crazyfile': open(path_to_wheel, 'rb')}

    files = {'crazyfile': open(wheel_package, 'rb')}
    r = requests.post(ASKANNA_FILEUPLOAD_ENDPOINT, files=files)

    if r.status_code == 201 or r.status_code == 200:
        click.echo("Upload complete!")
        click.echo("You can see your package in: ")
        askanna_location = r.json()
        _location = askanna_location.get('crazyfile')
        click.echo("\t{_location}".format(_location=_location))



"""
Management of variables in AskAnna
This is the class which act as gateway to the API of AskAnna
"""
from . import client
from dataclasses import dataclass
import datetime
import uuid
import sys


@dataclass
class Variable:
    name: str
    value: str
    short_uuid: str
    uuid: uuid.UUID
    is_masked: bool
    created: datetime.datetime
    modified: datetime.datetime
    project: dict


class VariableGateway:
    def __init__(self, *args, **kwargs):
        self.client = client
        self.project = None
        self.cache = {}
        self.base_url = self.client.config.remote + "variable/"

    def list(self, project=None, show_masked=False):
        url = self.base_url
        if project:
            # build url to select for project only
            url = "{}{}/{}/{}".format(
                self.client.config.remote,
                "project",
                project,
                "variables"
            )
        r = self.client.get(url)
        if r.status_code != 200:
            print("We cannot find variables for you")
            sys.exit(1)

        variables = [Variable(**var) for var in r.json()]
        self.cache = variables
        return variables

    def detail(self, short_uuid) -> dict:
        url = "{}{}/".format(self.base_url, short_uuid)
        r = self.client.get(url)
        return Variable(**r.json())

    def change(self, short_uuid, variable):
        url = "{}{}/".format(self.base_url, short_uuid)

        # emit update request (PATCH)
        r = self.client.patch(
            url,
            json={
                "name": variable.name,
                "value": variable.value,
                "is_masked": variable.is_masked
            }
        )

        # show success or failure
        if r.status_code == 200:
            variable_name = r.json().get('name')
            print("You have successfully changed the variable: '{variable_name}'".format(variable_name=variable_name))
            sys.exit(0)
        else:
            print("Something went wrong while changing the variable")
            sys.exit(1)
        return Variable(**r.json())

    def delete(self, short_uuid):
        url = "{}{}/".format(self.base_url, short_uuid)
        r = self.client.delete(url)
        return r.status_code == 204

    def create(self, name, value, masked, project):
        url = self.base_url
        r = self.client.create(url, json={
            "name": name,
            "value": value,
            "is_masked": masked,
            "project": project
        })
        if r.status_code == 201:
            return Variable(**r.json()), True

        return None, False



from askanna.cli.utils import get_config
from askanna.cli.core.user import User


class Config:
    """
    Configuration management for AskAnna CLI
    """

    def __init__(self, *args, **kwargs):
        self.config = get_config()
        self.user = self.user_from_config()

    @property
    def remote(self):
        return self.config.get("askanna", {}).get("remote", "https://api.askanna.eu/v1/")

    @property
    def auth(self):
        return self.config.get('auth', {})

    def user_from_config(self):
        """
        Try to read an auth token
        if that is the case configure a User instance with a token set
        """
        token = self.auth.get('token')
        return User(token=token)



import click
import pkg_resources


HELP = """
This command will allow you to perform a run on a remote
location for the particular project.
"""

SHORT_HELP = "Execute run of deployed AskAnna project"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.argument("kernel", required=False, default="default")
def cli(kernel):

    kernels = {}

    # inspect the askanna_kernel entry_point for existing kernels
    for entry_point in pkg_resources.iter_entry_points('askanna_kernels'):
        kernels[entry_point.name] = entry_point.load()

    if kernel != "default":

        # retrieve the passed on kernel, and execute
        try:
            torun = kernels[kernel]
        except KeyError:
            click.echo("No Kernel with name {kernel}".format(kernel=kernel))
            return 0

        torun.run_kernel()

    else:

        if kernels:
            click.echo("The following kernels are available:")
            for key, value in kernels.items():
                click.echo("\t{key}".format(key=key))

            click.echo("\nRun with:\n")
            click.echo("\taskanna run [kernel]")

        click.echo("No kernels yet registered in the system!")
        click.echo("Use the createproject to create some!")



import json
import logging
import os
import re
import sys
import tempfile
import uuid

import click
import git

from askanna import config_dirs
from askanna.cli.utils import zipFilesInDir, scan_config_in_path
from askanna.cli.utils import get_config, getProjectInfo, getProjectPackages
from askanna.cli.core.upload import PackageUpload

logging.basicConfig(format='%(levelname)s:%(message)s', level=logging.INFO)

HELP = """
Wrapper command to push the current working folder to archive.\n
Afterwards we send this to AskAnna
"""

SHORT_HELP = "Push code to AskAnna"


def package(src: str) -> str:

    pwd_dir_name = os.path.basename(src)
    random_suffix = uuid.uuid4().hex

    # make a temporary directory
    tmpdir = tempfile.mkdtemp(prefix="askanna-package")

    random_name = os.path.join(
        tmpdir, "{pwd_dir_name}_{random_suffix}.zip".format(
            pwd_dir_name=pwd_dir_name,
            random_suffix=random_suffix
        ))

    zipFilesInDir(src, random_name, lambda x: x)
    return random_name


def check_existing_package(project, push_target, upload_folder, api_server, token) -> bool:

    # check existence the usercache folder
    askanna_folder = config_dirs.user_cache_dir
    if not(os.path.exists(
        askanna_folder
    ) and os.path.isdir(askanna_folder)):
        packages = getProjectPackages(project, api_server, token)
        return len(packages) > 0
    # read contents from .askanna folder
    try:
        with open(os.path.join(
            askanna_folder,
            'PACKAGE-INFO'
        )) as f:
            j = json.loads(f.read())
    except Exception as e:
        # information is not there
        # please retrieve this info from askanna
        logging.debug(e)
        packages = getProjectPackages(project, api_server, token)
        return len(packages) > 0
    else:
        if push_target in j.keys():
            return True
        else:
            return False

    return True


def writePackageInfo(project, push_target, upload_folder):
    askanna_folder = config_dirs.user_cache_dir
    os.makedirs(askanna_folder, exist_ok=True)

    # does the file exist? create one if not

    try:
        with open(os.path.join(askanna_folder, 'PACKAGE-INFO')) as f:
            j = json.loads(f.read())
    except Exception:
        with open(os.path.join(askanna_folder, 'PACKAGE-INFO'), 'w') as f:
            f.write(json.dumps({
                push_target: project
            }, indent=2))
    else:
        with open(os.path.join(askanna_folder, 'PACKAGE-INFO'), 'w') as f:
            j.update(**{
                push_target: project
            })
            f.write(json.dumps(j, indent=2))


def extract_push_target(push_target):
    """
    Extract push target from the url configured
    Workspace is optional
    """
    match_pattern = re.compile(r"(?P<http_scheme>https|http):\/\/(?P<askanna_host>[\w\.\-\:]+)\/(?P<workspace_suuid>[\w-]+){0,1}\/{0,1}project\/(?P<project_suuid>[\w-]+)\/{0,1}")  # noqa
    matches = match_pattern.match(push_target)
    matches_dict = matches.groupdict()
    return matches_dict


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option('--force', '-f', is_flag=True, help='Force push')
@click.option('--message', '-m', default='', type=str, help='Add description to this code')
def cli(force, message):
    config = get_config()
    token = config['auth']['token']
    api_server = config['askanna']['remote']
    project = config.get('project', {})
    project_uuid = project.get('uuid')

    # read and parse the push-target from askanna
    push_target = config.get('push-target')

    if not push_target:
        print("`push-target` is not set, please set the `push-target` in order to push to AskAnna")
        sys.exit(1)

    matches_dict = extract_push_target(push_target)
    api_host = matches_dict.get("askanna_host")
    http_scheme = matches_dict.get("http_scheme")
    if api_host:
        # first also modify the first part
        if api_host.startswith('localhost'):
            api_host = api_host
        elif api_host not in ['askanna.eu']:
            first_part = api_host.split('.')[0]
            last_part = api_host.split('.')[1:]
            api_host = ".".join(
                [first_part+"-api"]+last_part
            )
        else:
            api_host = 'api.' + api_host
        api_server = "{}://{}/v1/".format(http_scheme, api_host)
    project_suuid = matches_dict.get("project_suuid")

    project_info = {}
    if project_suuid:
        # make an extra call to askanna to query for the full uuid for this project
        project_info = getProjectInfo(project_suuid, api_server, token)
        if project_info == {}:
            print("Couldn't find specified project {}".format(push_target))
            sys.exit(1)

        project_uuid = project_info.get('uuid')

    if not project_uuid:
        print("Cannot upload unregistered project to AskAnna")
        sys.exit(1)

    def ask_overwrite() -> bool:
        confirm = input("Do you want to replace the current code on AskAnna? [y/n]: ")
        answer = confirm.strip()
        if answer not in ['n', 'y']:
            print("Invalid option selected, choose from: y or n")
            return ask_overwrite()
        if confirm == 'y':
            return True
        else:
            return False

    cwd = os.getcwd()

    upload_folder = cwd
    askanna_config = scan_config_in_path()
    project_folder = os.path.dirname(askanna_config)

    def ask_which_folder(cwd, project_folder) -> str:
        confirm = input("Proceed upload [c]urrent or [p]roject folder? : ")
        answer = confirm.strip()
        if answer not in ['c', 'p']:
            print("Invalid option selected, choose from: c or p")
            return ask_which_folder(cwd, project_folder)
        if confirm == 'c':
            return cwd
        if confirm == 'p':
            return project_folder

    if not cwd == project_folder:
        print("You are not at the root folder of the project '{}'".format(project_folder))
        upload_folder = ask_which_folder(cwd, project_folder)

    # check for existing package
    if check_existing_package(project_info, push_target, upload_folder, api_server, token):
        # ask for confirmation if `-f` flag is not set
        overwrite = force
        if not force:
            overwrite = ask_overwrite()

        if not overwrite:
            print("We are not pushing your code to AskAnna. You choose not to replace your existing code.")
            sys.exit(0)

    package_archive = package(upload_folder)

    # attach message to this package upload
    if not message:
        # try git
        try:
            repo = git.Repo(".")
        except Exception as e:
            print(e)
        else:
            commit = repo.head.commit
            message = commit.message

    # if there is still no message set then use the zipfilename
    if not message:
        message = os.path.basename(package_archive)

    click.echo("Uploading '{}' to AskAnna...".format(upload_folder))

    fileinfo = {
        "filename": os.path.basename(package_archive),
        "size": os.stat(package_archive).st_size,
    }
    uploader = PackageUpload(
        token=token,
        api_server=api_server,
        project_uuid=project_uuid,
        description=message
    )
    status, msg = uploader.upload(package_archive, config, fileinfo)
    if status:
        print(msg)

        # create log that this package was uploaded once
        writePackageInfo(project_info, push_target, upload_folder)
        # remove temporary zipfile from the system (including the parent folder, this was the temp folder)
        try:
            os.remove(package_archive)
            os.rmdir(os.path.dirname(package_archive))
        except OSError as e:
            print("Error in removing the tmpfile of askanna-push: {}: {}".format(package_archive, e.strerror))
        sys.exit(0)
    else:
        print(msg)
        sys.exit(1)



import click
import time

HELP = """
Demonstrate how the interface can pontentially work, by faking requests
and responses.
"""


SHORT_HELP = "Mocking interactions with AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    verbose = 3
    print("**** performing askanna push ******")
    print("**** read .askanna (system) ******")
    time.sleep(50/1000)
    print("**** read ~/.askanna (user homedir) ******")
    time.sleep(50/1000)
    print("**** read .askanna (project) ******")
    time.sleep(50/1000)
    print("**** packaging current directoy ******")
    time.sleep(150/1000)
    print("**** create manifest ******")
    time.sleep(5/10000)
    print("**** upload manifest to askanna.io ******")
    time.sleep(150/1000)
    print("**** received upload token from askanna.io ******")
    time.sleep(250/1000)
    print("**** chunking package 1000 GB into 100 pieces *****")
    time.sleep(2000/1000)
    url = "https://sa080asdf098ads0f8a08asfd.afs.askanna.io/awesomeproject/upload"  # NOQA
    print("**** Start upload to {url} ****".format(url=url))
    for x in range(1, 100):
        if verbose > 2:
            print("**** uploading piece {x}/100 ".format(x=x))
        time.sleep(100/1000)
    print("**** upload completed *****")
    time.sleep(250/1000)
    print("**** Waiting for ack from askanna.io *****")
    time.sleep(500/1000)

    response_url = "https://askanna.io/awesomeproject/job/49283/status"
    print("**** [ASKANNA] Congratulations Bob! Your submission is well received ****")  # NOQA
    print("**** [ASKANNA] Access: {response_url} *****".format(
        response_url=response_url)
    )
    print("**** [ASKANNA] End of of message *****")



import click
import requests
import os
import yaml
from askanna.cli.utils import get_config

HELP = """
This command will allow you to create a project
"""

SHORT_HELP = "Create an AskAnna project"


def find_workspace(api_server: str, headers: dict) -> list:
    """
        Find all workspaces where the user is part of and enumerate them
    """
    workspaces = []
    r = requests.get('{}workspace'.format(api_server), headers=headers)

    for workspace in r.json():
        workspaces.append({
            'suuid': workspace['short_uuid'],
            'title': workspace['title']
        })
    for idx, workspace in enumerate(workspaces, start=1):
        print("%d. %s" % (idx, workspace['title']))
    return workspaces


class CreateProject:
    def __init__(self, name=None, push_target=None, project_info=None):
        self.name = name,
        self.push_target = push_target,
        self.project_info = project_info

    def cli(self):
        config = get_config()
        token = config['auth']['token']
        api_server = config['askanna']['remote']
        headers = {
            'Authorization': "Token {usertoken}".format(
                usertoken=token
            )
        }
        self.name = click.prompt(
            "Project name ",
            type=str
        )
        workspaces = find_workspace(api_server, headers)
        workspace = click.prompt(

            "Enter which workspace to use "
        )
        selected_workspace = workspaces[int(workspace)-1]

        self.project_info = requests.post(api_server+"project/", headers=headers, data={
            "name": self.name,
            "workspace": selected_workspace['suuid']
        })
        if self.project_info.status_code == 201:
            self.project_info = self.project_info.json()
            click.echo('You have successfully created a new project and connected your existing directory to AskAnna.')
            return self.name, self.project_info
        else:
            raise Exception("could not create project")

    def create_file(self):
        cwd = os.getcwd()
        project_info = self.project_info
        self.push_target = project_info['url']
        askanna_project_file = os.path.join(cwd, "askanna.yml")
        if not os.path.exists(askanna_project_file):
            with open(askanna_project_file, 'w') as pf:
                pf.write(yaml.dump({
                    "push_target": self.push_target,
                    "first job":
                        {"job": ""
                         }
                }, indent=2))


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    project_creator = CreateProject()
    project_name, project_info = project_creator.cli()

    if project_name:
        project_creator.create_file()
    click.echo("As a first step you can configure your first job for this project in the `askanna.yml` file. "
               "Ones you are done, you can easily push your code to askanna via:\n"
               "\n" "  askanna push\n" "\n"
               "Success with your project!")



# -*- coding: utf-8 -*-
import os
import sys
import click

from askanna.cli.utils import scan_config_in_path
from askanna.cli.utils import get_config
from askanna.cli.core.upload import ResultUpload

HELP = """
After a jobrun, we can add the jobresult to askanna
"""

SHORT_HELP = "Upload the result from a jobrun"


def create_jobresult(jobname: str, cwd: str) -> str:
    config = get_config()

    paths = config[jobname].get('output', {}).get('result')

    if isinstance(paths, list):
        print("Please enter a path in output/result, not a list")
        sys.exit(1)

    return paths


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    config = get_config()
    token = config['auth']['token']
    api_server = config['askanna']['remote']
    project = config.get('project', {})
    project_uuid = project.get('uuid')

    jobrun_uuid = os.getenv('JOBRUN_UUID')
    jobrun_suuid = os.getenv('JOBRUN_SUUID')
    jobrun_jobname = os.getenv('JOBRUN_JOBNAME')

    result_uuid = os.getenv('RESULT_UUID')
    result_suuid = os.getenv('RESULT_SUUID')

    # first check whether we need to create artifacts or not
    # if output is not specifed
    # or if output.paths is not specified
    # then we skip this step and report this to the stdout

    output_defined = config[jobrun_jobname].get('output')
    result_defined = config[jobrun_jobname].get('output', {}).get('result')

    if None in [output_defined, result_defined]:
        print("Result storage aborted, no `output` or `output/result` defined in `askanna.yml`")
        sys.exit(0)

    cwd = os.getcwd()

    upload_folder = cwd
    askanna_config = scan_config_in_path()
    project_folder = os.path.dirname(askanna_config)

    def ask_which_folder(cwd, project_folder) -> str:
        confirm = input("Proceed upload [c]urrent or [p]roject folder? : ")
        answer = confirm.strip()
        if answer not in ['c', 'p']:
            return ask_which_folder(cwd, project_folder)
        if confirm == 'c':
            return cwd
        if confirm == 'p':
            return project_folder

    if not cwd == project_folder:
        print("You are not at the root folder of the project '{}'".format(
            project_folder))
        upload_folder = ask_which_folder(cwd, project_folder)

    # First check whether we have a result defined
    paths = config[jobrun_jobname].get('output', {}).get('result')
    if not paths:
        print("No output was defined in output/result, not uploading to AskAnna")
        sys.exit(0)

    jobresult_archive = create_jobresult(jobname=jobrun_jobname, cwd=upload_folder)

    click.echo("Uploading result to AskAnna...")

    fileinfo = {
        "filename": os.path.basename(jobresult_archive),
        "size": os.stat(jobresult_archive).st_size,
    }
    uploader = ResultUpload(
        token=token,
        api_server=api_server,
        project_uuid=project_uuid,
        JOBRUN_UUID=jobrun_uuid,
        JOBRUN_SUUID=jobrun_suuid,
        RESULT_UUID=result_uuid,
        RESULT_SUUID=result_suuid,
    )
    status, msg = uploader.upload(jobresult_archive, config, fileinfo)
    if status:
        print(msg)
        sys.exit(0)
    else:
        print(msg)
        sys.exit(1)



# -*- coding: utf-8 -*-
from askanna.cli.utils import init_checks
from askanna.cli.utils import update_available
import askanna
import click
import importlib
from dotenv import find_dotenv, load_dotenv
load_dotenv(find_dotenv())


HELP = """
The AskAnna CLI helps you running data science projects on AskAnna.
"""

SHORT_HELP = "AskAnna CLI client"

EPILOG = """
For usage and help on a specific command, run it with a --help flag, e.g.:

    askanna login --help
"""

CONTEXT_SETTINGS = {'help_option_names': ['-h', '--help']}


@click.group(help=HELP, short_help=SHORT_HELP, epilog=EPILOG,
             context_settings=CONTEXT_SETTINGS)
@click.version_option(askanna.__version__)
def cli():
    update_url = update_available()
    if update_url:
        click.echo("INFO: A newer version of AskAnna is available. Update "
                   "via pip or get it at {}".format(update_url), err=True)


commands = [
    "login",
    "logout",
    "init",
    "createproject",
    "payload",
    "push",
    "package_download",
    "jobrun_manifest",
    "artifact",
    "upload_result",
    "variable",
]

for command in commands:
    module_path = "askanna.cli." + command
    command_module = importlib.import_module(module_path)
    command_name = command.replace('_', '-')
    cli.add_command(command_module.cli, command_name)


# perform initial checks on config and auth status
init_checks()



import io
import os
import resumable

from askanna.cli.core import client as askanna_client
from askanna.cli.utils import _file_type, diskunit


class Upload:
    tpl_register_upload_url = "{ASKANNA_API_SERVER}package/"
    tpl_register_chunk_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/packagechunk/"
    tpl_upload_chunk_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/packagechunk/{CHUNK_UUID}/chunk/"
    tpl_final_upload_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/finish_upload/"

    tpl_upload_pass = "uploaded"
    tpl_upload_fail = "failed"

    def __init__(self, token: str, api_server: str, project_uuid: str, *args, **kwargs):
        self.ASKANNA_API_SERVER = api_server
        self.suuid = None
        self.project_uuid = project_uuid
        self.resumable_file = None
        self.kwargs = kwargs

    @property
    def chunk_baseinfo(self) -> dict:
        return {
            'resumableChunkSize': self.resumable_file.chunk_size,
            'resumableTotalSize': self.resumable_file.size,
            'resumableType': _file_type(self.resumable_file.path),
            'resumableIdentifier': str(self.resumable_file.unique_identifier),
            'resumableFilename': os.path.basename(self.resumable_file.path),
            'resumableRelativePath': self.resumable_file.path,
            'resumableTotalChunks': len(self.resumable_file.chunks),
            'resumableChunkNumber': 1,
            'resumableCurrentChunkSize': 1
        }.copy()

    def url_template_arguments(self):
        """
        This method is meant to be overwritten by inheritors
        providing specific url templating variables
        """
        return {
            'ASKANNA_API_SERVER': self.ASKANNA_API_SERVER,
            'PACKAGE_SUUID': self.suuid
        }

    @property
    def register_upload_url(self) -> str:
        return self.tpl_register_upload_url.format(
            **self.url_template_arguments()
        )

    @property
    def register_chunk_url(self) -> str:
        return self.tpl_register_chunk_url.format(
            **self.url_template_arguments()
        )

    def upload_chunk_url(self, chunk_uuid: str) -> str:
        arguments = self.url_template_arguments()
        arguments.update({
            'CHUNK_UUID': chunk_uuid
        })
        return self.tpl_upload_chunk_url.format(
            **arguments
        )

    @property
    def final_upload_url(self) -> str:
        return self.tpl_final_upload_url.format(
            **self.url_template_arguments()
        )

    def upload(self, file_obj, config, fileinfo):
        self.create_entry(config=config, fileinfo=fileinfo)
        self.upload_file(file_obj)
        return self.finish_upload()

    def create_entry_extrafields(self):
        return {}

    def create_entry(self, config: dict = None, fileinfo: dict = {}) -> str:
        info_dict = {
            "filename": fileinfo.get('filename'),
            "project": self.project_uuid,
            "size": fileinfo.get('size')
        }

        info_dict.update(**self.create_entry_extrafields())

        # the request to AskAnna API
        req = askanna_client.post(
            self.register_upload_url,
            json=info_dict
        )
        res = req.json()

        # the result
        self.uuid = res.get('uuid')
        self.suuid = res.get('short_uuid')
        return self.suuid

    def upload_chunk(self, chunk, chunk_dict):
        config = chunk_dict.copy()
        config.update(**{
            "filename": chunk.index + 1,
            "size": chunk.size,
            "file_no": chunk.index + 1,
            "is_last": len(self.resumable_file.chunks) == chunk.index + 1
        })

        # request chunk id from API
        req_chunk = askanna_client.post(
            self.register_chunk_url,
            json=config
        )
        chunk_uuid = req_chunk.json().get('uuid')

        files = {
            'file': io.BytesIO(chunk.read())
        }
        data = self.chunk_baseinfo
        data.update(**{
            'resumableChunkNumber': chunk.index + 1,
            'resumableCurrentChunkSize': chunk.size
        })

        specific_chunk_req = askanna_client.post(
            self.upload_chunk_url(chunk_uuid=chunk_uuid),
            data=data,
            files=files
        )
        assert specific_chunk_req.status_code == 200, "Code could not be uploaded"

    def chunk_dict_template(self):
        return {
            "filename": "",
            "size": 0,
            "file_no": 0,
            "is_last": False,
            "package": self.uuid,
        }

    def upload_file(self, file_obj):
        """
        Take a file_obj and make chunks out of it to upload
        """
        chunk_dict = self.chunk_dict_template()

        self.resumable_file = resumable.file.ResumableFile(
            file_obj, 1*diskunit.MiB)
        for chunk in self.resumable_file.chunks:
            self.upload_chunk(chunk, chunk_dict)

    def finish_upload(self):
        # Do final call when all chunks are uploaded
        final_call_dict = self.chunk_baseinfo

        final_call_req = askanna_client.post(
            self.final_upload_url,
            data=final_call_dict
        )

        if final_call_req.status_code == 200:
            # print(final_call_req.text)
            return True, self.tpl_upload_pass
        else:
            return False, self.tpl_upload_fail


class PackageUpload(Upload):
    tpl_register_upload_url = "{ASKANNA_API_SERVER}package/"
    tpl_register_chunk_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/packagechunk/"
    tpl_upload_chunk_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/packagechunk/{CHUNK_UUID}/chunk/"
    tpl_final_upload_url = "{ASKANNA_API_SERVER}package/{PACKAGE_SUUID}/finish_upload/"

    tpl_upload_pass = "Package is uploaded"
    tpl_upload_fail = "Package upload failed"

    def create_entry_extrafields(self):
        return {
            'title': self.kwargs.get('description', "")[:50],
            'description': self.kwargs.get('description')
        }


class ArtifactUpload(Upload):
    tpl_register_upload_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/artifact/"
    tpl_register_chunk_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/artifact/{ARTIFACT_SUUID}/artifactchunk/"
    tpl_upload_chunk_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/artifact/{ARTIFACT_SUUID}/artifactchunk/{CHUNK_UUID}/chunk/"  # noqa
    tpl_final_upload_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/artifact/{ARTIFACT_SUUID}/finish_upload/"

    tpl_upload_pass = "Artifact is uploaded"
    tpl_upload_fail = "Artifact upload failed"

    def url_template_arguments(self):
        """
        This method is meant to be overwritten by inheritors
        providing specific url templating variables
        """
        return {
            'ASKANNA_API_SERVER': self.ASKANNA_API_SERVER,
            'ARTIFACT_SUUID': self.suuid,
            'JOBRUN_SUUID': self.kwargs.get('JOBRUN_SUUID')
        }

    def chunk_dict_template(self):
        return {
            "filename": "",
            "size": 0,
            "file_no": 0,
            "is_last": False,
            "artifact": self.uuid,
        }


class ResultUpload(Upload):
    tpl_register_upload_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/result/"
    tpl_register_chunk_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/result/{RESULT_SUUID}/resultchunk/"
    tpl_upload_chunk_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/result/{RESULT_SUUID}/resultchunk/{CHUNK_UUID}/chunk/"  # noqa
    tpl_final_upload_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/result/{RESULT_SUUID}/finish_upload/"

    tpl_upload_pass = "Result is uploaded"
    tpl_upload_fail = "Result upload failed"

    def url_template_arguments(self):
        return {
            'ASKANNA_API_SERVER': self.ASKANNA_API_SERVER,
            'JOBRUN_SUUID': self.kwargs.get('JOBRUN_SUUID'),
            'RESULT_UUID': self.uuid,
            'RESULT_SUUID': self.kwargs.get('RESULT_SUUID')
        }

    def chunk_dict_template(self):
        return {
            "filename": "",
            "size": 0,
            "file_no": 0,
            "is_last": False,
            "joboutput": self.uuid,
        }

    def create_entry(self, config: dict = None, fileinfo: dict = {}) -> str:
        """
        We don't have to create a new entry as the RESULT_UUID is already defined in the enviroment
        """
        # the result
        self.uuid = os.getenv('RESULT_UUID')
        return self.uuid



import os

import click
from zipfile import ZipFile

from askanna.cli.core import client as askanna_client
from askanna.cli.utils import get_config

HELP = """
Package downloader, intended for use with runner and unpacks code to /code
"""

SHORT_HELP = "Download package code for askanna"


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    config = get_config()
    api_server = config['askanna']['remote']
    project_suuid = os.getenv('PROJECT_SUUID')
    package_suuid = os.getenv('PACKAGE_SUUID')

    download_url = "/".join([
        'project',
        project_suuid, 'packages',
        package_suuid, 'download', ''])
    download_url = api_server + download_url

    r = askanna_client.get(download_url)
    res = r.json()

    r = askanna_client.get(res['target'])
    with open('/tmp/code.zip', 'wb') as f:
        f.write(r.content)

    with ZipFile('/tmp/code.zip', 'r') as myzip:
        myzip.extractall('/code')



# -*- coding: utf-8 -*-

"""Top-level package for askanna."""

__author__ = """Anthony Leung"""
__email__ = "anthony@askanna.io"
__version__ = "0.4.1"

try:
    from appdirs import AppDirs
except Exception as e:
    # we are propably within an installation
    print(e)
else:
    appname = "askanna"
    appauthor = "askanna"
    config_dirs = AppDirs(appname, appauthor)



import click
from cookiecutter.main import cookiecutter

# FIXME: this will be an AskAnna public link in the future
# For dev purposes, add your local path to the cookiecutter template
ASKANNA_BASEDSP = "https://gitlab.askanna.io/askanna/cookiecutter-askannabasedsp.git"
ASKANNA_BASEDSP = "git@gitlab.askanna.io:askanna/cookiecutter-askannabasedsp.git"

HELP = """
This command will allow you to create a project
"""

SHORT_HELP = "Create an AskAnna project"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.argument("template", required=False, default="basedsp")
def cli(template):
    click.echo("This is performing the createproject action")

    if template and template == "basedsp" or "base":
        cookiecutter(ASKANNA_BASEDSP)



# -*- coding: utf-8 -*-
import os
import sys

import click

from askanna.cli.core import client as askanna_client
from askanna.cli.utils import get_config
# read defaults from the environment
default_jobrun_suuid = os.getenv('JOBRUN_SUUID')
default_payload_suuid = os.getenv('PAYLOAD_SUUID')
default_payload_file = os.getenv('PAYLOAD_PATH')


HELP = """
Download the payload stored in AskAnna
"""

SHORT_HELP = "Get payload from AskAnna"


@click.command(help=HELP, short_help=SHORT_HELP)
@click.option('--jobrun', '-j', default=default_jobrun_suuid, show_default=True)
@click.option('--payload', '-p', default=default_payload_suuid, show_default=True)
@click.option('--output', '-o', default=default_payload_file, show_default=True,
              type=click.Path())
def cli(jobrun, payload, output):
    config = get_config()
    ASKANNA_API_SERVER = config['askanna']['remote']

    # we assume we can get jobrun id and the payload
    payload_url = "{ASKANNA_API_SERVER}jobrun/{JOBRUN_SUUID}/payload/{PAYLOAD_SUUID}/".format(
        JOBRUN_SUUID=jobrun,
        PAYLOAD_SUUID=payload,
        ASKANNA_API_SERVER=ASKANNA_API_SERVER
    )

    req = askanna_client.get(
        payload_url
    )

    if not req.status_code == 200:
        print("Could't retrieve payload {}".format(payload))
        sys.exit(1)

    if output:
        with open(output, 'w') as f:
            f.write(req.text)
    else:
        print(req.text)



from .base import BaseCLItest
from askanna.cli.push import extract_push_target


class TestCliPush(BaseCLItest):
    """
    askanna push

    We expect to initiate a push action of our code to the askanna server
    """
    verb = "push"

    def testCommandLoginBase(self):
        assert "push" in self.result.output
        self.assertIn("push", self.result.output)
        self.assertNotIn("noop", self.result.output)

    def testPushTarget(self):
        targets = [
            ['https://beta.askanna.eu/6swz-ujcr-jQQw-SAdZ/project/6mxc-03Ew-THeb-ftGG', {
                'askanna_host': 'beta.askanna.eu',
                'http_scheme': 'https',
                'project_suuid': '6mxc-03Ew-THeb-ftGG',
                'workspace_suuid': '6swz-ujcr-jQQw-SAdZ'
            }],
            ['https://beta.askanna.eu/6swz-ujcr-jQQw-SAdZ/project/6mxc-03Ew-THeb-ftGG/', {
                'askanna_host': 'beta.askanna.eu',
                'http_scheme': 'https',
                'project_suuid': '6mxc-03Ew-THeb-ftGG',
                'workspace_suuid': '6swz-ujcr-jQQw-SAdZ'
            }],
            ['https://beta.askanna.eu/project/6mxc-03Ew-THeb-ftGG', {
                'askanna_host': 'beta.askanna.eu',
                'http_scheme': 'https',
                'project_suuid': '6mxc-03Ew-THeb-ftGG',
                'workspace_suuid': None
            }],
            ['https://beta.askanna.eu/project/6mxc-03Ew-THeb-ftGG/', {
                'askanna_host': 'beta.askanna.eu',
                'http_scheme': 'https',
                'project_suuid': '6mxc-03Ew-THeb-ftGG',
                'workspace_suuid': None
            }],
            ['http://beta.askanna.eu/project/6mxc-03Ew-THeb-ftGG/', {
                'askanna_host': 'beta.askanna.eu',
                'http_scheme': 'http',
                'project_suuid': '6mxc-03Ew-THeb-ftGG',
                'workspace_suuid': None
            }],
        ]
        for target in targets:
            self.assertDictEqual(extract_push_target(target[0]), target[1])



# -*- coding: utf-8 -*-
import math
import os
import sys
import time
from zipfile import ZipFile
import click

from askanna.cli.core import client as askanna_client
from askanna.cli.utils import zipPaths
from askanna.cli.utils import scan_config_in_path
from askanna.cli.utils import get_config, string_expand_variables
from askanna.cli.core.upload import ArtifactUpload


@click.group()
def cli1():
    pass


@click.group()
def cli2():
    pass


def create_artifact(jobname: str, cwd: str) -> str:
    config = get_config()

    paths = config[jobname].get('output', {}).get('paths')

    # expand and translate paths if they are configured with variables
    paths = string_expand_variables(paths)

    zipFileName = '/tmp/artifact.zip'
    with ZipFile(zipFileName, mode='w') as zipObj:
        zipPaths(zipObj, paths, cwd)

    return zipFileName


@cli1.command(help="After a run of a job, we can save job run artifacts to an archive",
              short_help="Save artifact from a job run")
def add():
    config = get_config()
    token = config['auth']['token']
    api_server = config['askanna']['remote']

    project_uuid = os.getenv('PROJECT_UUID')
    project_suuid = os.getenv('PROJECT_SUUID')

    jobrun_suuid = os.getenv('JOBRUN_SUUID')
    jobrun_jobname = os.getenv('JOBRUN_JOBNAME')

    if not project_suuid:
        print("Cannot upload unregistered project to AskAnna")
        sys.exit(1)

    # first check whether we need to create artifacts or not
    # if output is not specifed
    # or if output.paths is not specified
    # then we skip this step and report this to the stdout

    output_defined = config[jobrun_jobname].get('output')
    paths_defined = config[jobrun_jobname].get('output', {}).get('paths')

    if None in [output_defined, paths_defined]:
        print("Artifact creation aborted, no `output` or `output/paths` defined in `askanna.yml`")
        sys.exit(0)

    cwd = os.getcwd()

    upload_folder = cwd
    askanna_config = scan_config_in_path()
    project_folder = os.path.dirname(askanna_config)

    def ask_which_folder(cwd, project_folder) -> str:
        confirm = input("Proceed upload [c]urrent or [p]roject folder? : ")
        answer = confirm.strip()
        if answer not in ['c', 'p']:
            return ask_which_folder(cwd, project_folder)
        if confirm == 'c':
            return cwd
        if confirm == 'p':
            return project_folder

    if not cwd == project_folder:
        print("You are not at the root folder of the project '{}'".format(
            project_folder))
        upload_folder = ask_which_folder(cwd, project_folder)

    artifact_archive = create_artifact(jobname=jobrun_jobname, cwd=upload_folder)

    click.echo("Uploading artifact to AskAnna...")

    fileinfo = {
        "filename": os.path.basename(artifact_archive),
        "size": os.stat(artifact_archive).st_size,
    }
    uploader = ArtifactUpload(
        token=token,
        api_server=api_server,
        project_uuid=project_uuid,
        JOBRUN_SUUID=jobrun_suuid
    )
    status, msg = uploader.upload(artifact_archive, config, fileinfo)
    if status:
        print(msg)
        sys.exit(0)
    else:
        print(msg)
        sys.exit(1)


class ChunkedDownload:
    def __init__(self, url, client, output=None):
        """
        Takes an url to download from, this can be a url which could redirect to another URI.
        """
        self.history = []
        self.download_queue = []
        self.chunk_size = 1024 * 1024 * 100  # 100MB
        self.output_target = output

        self.url = url
        self.size = 0
        self.accept_ranges = 'none'

        self.client = client
        self.perform_preflight(url=url)

    @property
    def status_code(self):
        """
        Return the last status_code
        """
        return self.history[-1][1]

    def perform_preflight(self, url):
        """
        We take the session from self.client and will do a preflight request to
        determine whether the url will result in a (final) http_response_code=200
        """
        r = self.client.head(url)
        self.history.append([url, r.status_code, r.headers])

        if r.status_code in [301, 302] and r.headers.get('Location'):
            self.url = r.headers.get('Location')
            return self.perform_preflight(url=r.headers.get('Location'))
        elif r.status_code == 200:
            self.size = int(r.headers.get('Content-Length'))
            self.accept_ranges = r.headers.get('Accept-Ranges', 'none')

    def setup_download(self):
        """
        Download the self.url and chunk (if needed) the download
        """
        no_chunks = math.ceil(self.size / self.chunk_size)
        for chunk_no in range(0, no_chunks):
            self.download_queue.append([chunk_no, chunk_no * self.chunk_size, (chunk_no + 1) * self.chunk_size - 1])
        # fix last end byte
        self.download_queue[-1][-1] = self.size

    def download(self):
        """
        Download the target_url, we consume the queue and write partial files.
        In the end we glue them to the original intended file.
        """
        self.setup_download()
        finished_queue = []
        while len(self.download_queue):
            chunk = self.download_queue.pop(0)
            range_header = {
                'Range': "{accept_ranges}={start}-{end}".format(
                    accept_ranges=self.accept_ranges,
                    start=chunk[1],
                    end=chunk[2]
                )
            }

            try:
                r = self.client.get(self.url, stream=True, headers=range_header)
                # so far so good on the download, save the result
                with open('file_{}.part'.format(chunk[0]), 'wb') as f:
                    for dlchunk in r.iter_content(chunk_size=1024):
                        f.write(dlchunk)
            except Exception as e:
                print(e)
                self.download_queue.append(chunk)
            else:
                finished_queue.append(chunk)
        # combine the chunks in to one file
        with open(self.output_target, 'wb') as f:
            finished_queue = sorted(finished_queue, key=lambda x: x[0])
            for chunk in finished_queue:
                chunkfilename = 'file_{}.part'.format(chunk[0])
                with open(chunkfilename, 'rb') as chunkfile:
                    f.write(chunkfile.read())
                # delete the chunkfile
                os.remove(chunkfilename)


@cli2.command(help="Download an artifact of a job run", short_help="Download a job run artifact")
@click.option('--id', '-i', required=True, type=str, help='Job run SUUID')
@click.option('--output', '-o', show_default=True, type=click.Path())
def get(id, output):
    """
    Download an artifact of a job run
    """
    config = get_config()
    ASKANNA_API_SERVER = config.get('askanna', {}).get('remote')

    base_url = "{server}".format(server=ASKANNA_API_SERVER)
    url = base_url + "artifact/{}".format(id)

    if not output:
        output = "artifact_{suuid}_{datetime}.zip".format(suuid=id, datetime=time.strftime("%Y%m%d-%H%M%S"))

    if os.path.exists(output):
        print("The output file already exists. We will not overwrite the existing file.")
        sys.exit(1)

    stable_download = ChunkedDownload(url=url,
                                      client=askanna_client,
                                      output=output)
    if stable_download.status_code != 200:
        print("We cannot find this artifact for you")
        sys.exit(1)
    print("Downloading the artifact has started.")
    stable_download.download()

    print("We have succesfully downloaded the job run artifact.")
    print("The artifact is saved in: {file}".format(file=output))


cli = click.CommandCollection(sources=[cli1, cli2], help="Save and download job run artifacts",
                              short_help="Save and download job run artifacts")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages
from askanna import __version__ as askanna_version

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.md') as history_file:
    history = history_file.read()

requirements = [
    'Click>=7.1',
    'cookiecutter>=1.6.0',
    'requests>=2.22.0',
    'PyYAML>=5.3',
    'python-dotenv>=0.14.0',
    'resumable>=0.1.1',
    'gitpython>=3.1.3',
    'appdirs>=1.4.4',
    'dataclasses; python_version=="3.6.*"'
]

setup_requirements = [
    'wheel~=0.34.2'
]

test_requirements = []

setup(
    name='askanna',
    version=askanna_version,
    author="AskAnna",
    author_email='devops@askanna.io',
    python_requires='>=3.6',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        "Topic :: Scientific/Engineering",
    ],
    entry_points={
        'console_scripts': [
            'askanna=askanna.cli.tool:cli',
        ],
    },
    install_requires=requirements,
    description="AskAnna CLI is part of the AskAnna platform to kickstart your data science projects",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    include_package_data=True,
    keywords='askanna ml ai data',
    license="Apache License 2.0",
    packages=find_packages(exclude=['tests']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://askanna.io',
    project_urls={
        "Documentation": "https://docs.askanna.io/#/cli",
        "Source Code": "https://gitlab.askanna.io/askanna/askanna-cli",
    },
    zip_safe=False,
)




import collections
import glob
import mimetypes
import os
import re

from pathlib import Path
from zipfile import ZipFile

import requests
from yaml import load, dump
try:
    from yaml import CLoader as Loader, CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

CONFIG_USERHOME_FILE = "~/.askanna.yml"

StorageUnit = collections.namedtuple('StorageUnit',
                                     [
                                         'B', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB'
                                     ])

diskunit = StorageUnit(B=1, KiB=1024**1, MiB=1024**2, GiB=1024**3, TiB=1024**4, PiB=1024**5)


def init_checks():
    create_config(CONFIG_USERHOME_FILE)


def create_config(location: str):
    expanded_path = os.path.expanduser(location)
    folder = os.path.dirname(expanded_path)

    if not os.path.exists(folder):
        os.makedirs(folder, exist_ok=True)

    if not os.path.exists(expanded_path):
        Path(expanded_path).touch()

        # write initial config since it didn't exist
        with open(expanded_path, 'w') as f:
            config = store_config({
                'askanna': {
                    'remote': 'https://beta-api.askanna.eu/v1/'
                }
            })
            f.write(config)


def update_available(silent_fail=True):
    """
    Check whether most recent Gitlab release of askanna is newer than the
    askanna version in use. If a newer version is available, return a
    link to the release on Gitlab, otherwise return ``None``.
    """
    try:
        # FIXME: some code to check the release on Gitlab
        a = None
        return a
    except Exception:
        if not silent_fail:
            raise

        # Don't let this interfere with askanna usage
        return None


def check_for_project():
    """
    Performs a check if we are operating within a project folder. When
    we wish to perform a deploy action, we want to be on the same
    level with the ``askanna.yml`` to be able to package the file.
    """
    pyfiles = glob.glob('*.yml')

    # look for the setup.py file
    if 'askanna.yml' in pyfiles:
        return True
    else:
        return False


def scan_config_in_path(cwd=None):
    """
    Look for askanna.yml in parent directories
    """
    if not cwd:
        cwd = os.getcwd()
    project_configfile = ""
    # first check whether we already can find in the current workdir
    if contains_configfile(cwd):
        project_configfile = os.path.join(cwd, "askanna.yml")
    else:
        # traverse up all directories untill we find an askanna.yml file
        split_path = os.path.split(cwd)
        # in any other cases, look in parent directories
        while split_path[1] != "":
            if contains_configfile(split_path[0]):
                project_configfile = os.path.join(
                    split_path[0],
                    "askanna.yml"
                )
                break
            split_path = os.path.split(split_path[0])
    return project_configfile


def read_config(path: str) -> dict:
    """
    Reading existing config or return default dict
    """
    return load(open(os.path.expanduser(path), 'r'), Loader=Loader) or {}


def contains_configfile(path: str, filename: str = "askanna.yml") -> bool:
    return os.path.isfile(
        os.path.join(path, filename)
    )


def get_config() -> dict:
    init_checks()
    config = read_config(CONFIG_USERHOME_FILE) or {}
    project_config = scan_config_in_path()
    if project_config:
        config.update(**read_config(project_config))

    # overwrite the AA remote if AA_REMOTE is set in the environment
    is_remote_set = os.getenv('AA_REMOTE')
    if is_remote_set:
        config['askanna'] = config.get('askanna', {})
        config['askanna']['remote'] = is_remote_set

    # overwrite the user token if AA_TOKEN is set in the environment
    is_token_set = os.getenv('AA_TOKEN')
    if is_token_set:
        config['auth'] = config.get('auth', {})
        config['auth']['token'] = is_token_set

    # overwrite the project token if set in the env
    is_project_set = os.getenv('PROJECT_UUID')
    if is_project_set:
        config['project'] = config.get('project', {})
        config['project']['uuid'] = is_project_set

    return config


def store_config(new_config):
    original_config = get_config()
    original_config.update(**new_config)
    output = dump(original_config, Dumper=Dumper)
    return output


# Zip the files from given directory that matches the filter
def zipFilesInDir(dirName, zipFileName, filter):
    os.chdir(dirName)
    # create a ZipFile object
    with ZipFile(zipFileName, mode='w') as zipObj:
        # Iterate over all the files in directory
        for folderName, subfolders, filenames in os.walk('.'):
            for filename in filenames:
                if filter(filename):
                    # create complete filepath of file in directory
                    filePath = os.path.join(folderName, filename)
                    # Add file to zip
                    zipObj.write(filePath)


def zipAFile(zipObj: ZipFile, dirpath: str, filepath: str, prefixdir: str) -> None:
    # create complete filepath of file in directory
    filePath = os.path.join(dirpath, filepath)
    # Add file to zip
    zipObj.write(filePath)


def zipFolder(zipObj: ZipFile, dirpath: str, prefixdir: str):
    os.chdir(dirpath)
    # Iterate over all the files in directory
    for folderName, subfolders, filenames in os.walk('.'):
        for filename in filenames:
            # create complete filepath of file in directory
            filePath = os.path.join(folderName, filename)
            # Add file to zip
            zipObj.write(filePath, os.path.join(prefixdir, filePath))


def zipPaths(zipObj: ZipFile, paths: list, cwd: str):
    for targetloc in paths:
        if targetloc.startswith('/'):
            prefixdir = targetloc
        else:
            prefixdir = targetloc
            targetloc = os.path.join(cwd, targetloc)

        # check for existence

        if not os.path.exists(targetloc):
            print(targetloc, "does not exists ... skipping")
            continue

        if os.path.isdir(targetloc):
            zipFolder(zipObj, targetloc, prefixdir=prefixdir)
        else:
            # we got a file?
            zipAFile(zipObj, dirpath=os.path.dirname(targetloc),
                     filepath=os.path.basename(targetloc), prefixdir=prefixdir)


def _file_type(path):
    """Mimic the type parameter of a JS File object.
    Resumable.js uses the File object's type attribute to guess mime type,
    which is guessed from file extention accoring to
    https://developer.mozilla.org/en-US/docs/Web/API/File/type.
    Parameters
    ----------
    path : str
        The path to guess the mime type of
    Returns
    -------
    str
        The inferred mime type, or '' if none could be inferred
    """
    type_, _ = mimetypes.guess_type(path)
    # When no type can be inferred, File.type returns an empty string
    return '' if type_ is None else type_


def string_expand_variables(strings: list) -> list:
    var_matcher = re.compile(r"\$\{(?P<MYVAR>[\w\-]+)\}")
    for idx, line in enumerate(strings):
        matches = var_matcher.findall(line)
        for m in matches:
            line = line.replace("${"+m+"}", os.getenv(m.strip()))
        strings[idx] = line
    return strings


def getProjectInfo(project_suuid, api_server, token):
    r = requests.get(
        "{api_server}project/{project_suuid}/".format(
            api_server=api_server,
            project_suuid=project_suuid
        ),
        headers={
            'user-agent': 'askanna-cli/0.2.0',
            'Authorization': 'Token {token}'.format(
                token=token
            )
        }
    )
    if not r.status_code == 200:
        return {}

    return r.json()


def getProjectPackages(project, api_server, token, offset=0, limit=1):
    r = requests.get(
        "{api_server}project/{project_suuid}/packages/?offset={offset}&limit={limit}".format(
            api_server=api_server,
            project_suuid=project['short_uuid'],
            offset=offset,
            limit=limit
        ),
        headers={
            'user-agent': 'askanna-cli/0.2.0',
            'Authorization': 'Token {token}'.format(
                token=token
            )
        }
    )
    if not r.status_code == 200:
        return []

    return r.json()



import os

import click
import requests

from askanna.cli.utils import get_config

HELP = """
JobRun Manifest downloader intended to use in askanna-runner
"""

SHORT_HELP = "Download manifest for jobrun"


@click.option('--output', '-o', default='/entrypoint.sh', show_default=True,
              type=click.Path())
@click.command(help=HELP, short_help=SHORT_HELP)
def cli(output):
    config = get_config()
    token = config['auth']['token']
    api_server = config['askanna']['remote']
    jobrun_suuid = os.getenv("JOBRUN_SUUID")

    download_url = "/".join([
        'jobrun',
        jobrun_suuid, 'manifest', ''])
    download_url = api_server + download_url

    headers = {
        'user-agent': 'askanna-cli/0.3.0',
        'Authorization': 'Token {token}'.format(
            token=token
        )
    }

    r = requests.get(download_url, headers=headers)

    with open(output, 'w') as f:
        f.write(r.text)



import askanna
from askanna.cli.core.config import Config
from askanna.cli.core.session import Session


class Client:
    """
    Client for communication with AskAnna API service
    """

    def __init__(self, headers=None, *args, **kwargs):
        # TODO read config

        self.config = Config()
        self.headers = headers or self.generate_authenication_header()

        self.session = Session(headers=self.headers)

    def generate_authenication_header(self):
        if not self.config.user.token:
            return {}

        return {
            'user-agent': 'askanna-cli/{version}'.format(version=askanna.__version__),
            'Authorization': 'Token {token}'.format(
                token=self.config.user.token
            )
        }

    def get(self, url, **kwargs):
        return self.session.get(url, **kwargs)

    def head(self, url, **kwargs):
        return self.session.head(url, **kwargs)

    def patch(self, url, **kwargs):
        return self.session.patch(url, **kwargs)

    def post(self, url, **kwargs):
        return self.session.post(url, **kwargs)

    def create(self, url, **kwargs):
        return self.session.create(url, **kwargs)

    def delete(self, url, **kwargs):
        return self.session.delete(url, **kwargs)



import sys

import click

from askanna.cli.core.variables import VariableGateway

HELP = """
Manage your variables in AskAnna
"""

SHORT_HELP = "Manage variables in AskAnna"


@click.group()
def cli1():
    pass


@click.group()
def cli2():
    pass


@click.group()
def cli3():
    pass


@click.group()
def cli4():
    pass


@cli1.command(help="List variables in AskAnna", short_help="List variables")
@click.option('--project', '-p', required=False, type=str, help='Project SUUID to list variables for a project')
def list(project):
    """
    List variables
    """
    variable_gateway = VariableGateway()
    variables = variable_gateway.list(project=project)
    if not variables:
        print("Based on the information provided, we cannot find any variables.")
        sys.exit(0)
    if project:
        print("The variables for project \"{}\" are:\n".format(variables[0].project['name']))
        print("VARIABLE SUUID         VARIABLE NAME")
        print("-------------------    -------------------------")
    else:
        print("PROJECT SUUID          PROJECT NAME       VARIABLE SUUID         VARIABLE NAME")
        print("-------------------    ---------------    -------------------    -------------------------")
    for var in sorted(variables, key=lambda x: (x.project['name'], x.name)):
        if project:
            print("{suuid}    {variable_name}".format(
                suuid=var.short_uuid,
                variable_name=var.name))
        else:
            print("{project_suuid}    {project}    {suuid}    {variable_name}".format(
                project_suuid=var.project['short_uuid'],
                project="{:15}".format(var.project['name']),
                suuid=var.short_uuid,
                variable_name=var.name))


@cli2.command(help="Change a variable in AskAnna", short_help="Change variable")
@click.option('--id', '-i', required=True, type=str, help='Variable SUUID')
@click.option('--name', '-n', required=False, type=str, help='New name to set')
@click.option('--value', '-v', required=False, type=str, help='New value to set')
@click.option('--masked', '-m', required=False, type=bool, help='Set value to masked')
def change(id, name, value, masked):
    """
    Change a variable name, value and if the value should set to be masked.
    We will only proceed when any of the name or value is set.
    """
    short_uuid = id

    variable_gateway = VariableGateway()
    variable = variable_gateway.detail(short_uuid)

    if not any([name, value]):
        print("We did not change anything because you did not request to change a name or value.\n"
              "Please add additional input to the command to change the name or value of the variable.")
        sys.exit(0)
    if masked and variable.is_masked:
        print("This variable is currently masked. It is not possible to unmask a masked variable."
              "We skip the option to change the mask status of the variable.")
    # change the details of the variable
    variable.name = name or variable.name
    variable.value = value or variable.value
    variable.is_masked = masked or variable.is_masked

    # commit the change of the variable to AskAnna
    variable = variable_gateway.change(short_uuid, variable)


@cli3.command(help="Delete a variable in AskAnna", short_help="Delete variable")
@click.option('--id', '-i', required=True, type=str, help='Job variable SUUID')
@click.option('--force', '-f', is_flag=True, help='Force')
def delete(id, force):
    """
    Delete a variable in AskAnna
    """
    short_uuid = id
    variable_gateway = VariableGateway()
    try:
        variable = variable_gateway.detail(short_uuid)
    except TypeError:
        print(
            "It seems that a variable {id} doesn't exist.".format(id=id)
        )
        sys.exit(1)

    def ask_confirmation() -> bool:
        confirm = input("Are you sure to delete variable {id} with name \"{name}\"? [y/n]: ".format(
            id=id, name=variable.name
        ))
        answer = confirm.strip()
        if answer not in ['n', 'y']:
            print("Invalid option selected, choose from y or n")
            return ask_confirmation()
        if confirm == 'y':
            return True
        else:
            return False

    if not force:
        confirmation = ask_confirmation()
        if not confirmation:
            print("Understood. We are not deleting the variable.")
            sys.exit(0)

    variable_gateway = VariableGateway()
    deleted = variable_gateway.delete(short_uuid)
    if deleted:
        print("You deleted variable {id}".format(id=id))
    else:
        print("Something went wrong, deletion not performed.")


@cli4.command(help="Create a variable for a project in AskAnna", short_help="Create variable")
@click.option('--name', '-n', required=True, type=str, help='Name of the variable')
@click.option('--value', '-v', required=True, type=str, help='Value of the variable')
@click.option('--masked', '-m', required=False, type=bool, default=False, help='Set value to masked (default is False)')
@click.option('--project', '-p', required=True, type=str, help='Project SUUID where this variable will be created')
def add(name, value, masked, project):
    """
    Add a variable to a project
    """
    variable_gateway = VariableGateway()
    # Add variable to askanna
    variable, created = variable_gateway.create(name, value, masked, project)
    if created:
        print("You created variable \"{name}\" with SUUID {id}.".format(
            name=variable.name, id=variable.short_uuid))
        sys.exit(0)
    else:
        print("Something went wrong in creating the variable.")
        sys.exit(1)


cli = click.CommandCollection(sources=[cli1, cli2, cli3, cli4], help=HELP,
                              short_help=SHORT_HELP)



import getpass
import sys
import os

import click

from askanna.cli.core import client as askanna_client
from askanna.cli.utils import get_config, store_config

HELP = """
Add your AskAnna API key to your global configuration file
(~/.askanna.yml). This is necessary to gain access to projects associated with
your AskAnna account.
"""

SHORT_HELP = "Login and save your AskAnna API key"


def login(server):
    url = "{server}rest-auth/login/".format(server=server.replace("v1/", ''))
    print("Please provide your credentials to log into AskAnna")
    username = input("Username: ")
    password = getpass.getpass("Password: ")
    login_dict = {
        'username': username.strip(),
        'password': password.strip()
    }

    r = askanna_client.post(url, json=login_dict)
    if r.status_code == 200:
        token = r.json().get('key')
        return str(token)
    return None


def get_user_info(token, server):

    url = "{server}rest-auth/user".format(server=server.replace("v1/", ''))
    ruser = askanna_client.get(url)
    if ruser.status_code == 200:
        res = ruser.json()
        print("{} {}".format(res['first_name'], res['last_name']))
    else:
        print("Could not connect to AskAnna at this moment")


def do_login(server):
    """
    Pipeline of actions to do login and store the config into local config
    """
    token = login(server=server)
    new_config = {
        'auth': {
            'token': token
        }
    }
    config = store_config(new_config)
    with open(os.path.expanduser("~/.askanna.yml"), "w") as fd:
        fd.write(config)


@click.command(help=HELP, short_help=SHORT_HELP)
def cli():
    config = get_config()

    ASKANNA_API_SERVER = config.get('askanna', {}).get('remote')

    click.echo("Login into AskAnna")

    if config.get('auth'):
        if config['auth'].get('token'):
            click.echo("You are already logged in")
            # validate_token()
            token = config['auth']['token']
            # Showcase with this token, who we are
            get_user_info(token, server=ASKANNA_API_SERVER)
            sys.exit(0)
        else:
            # click.echo("Logging in for you")
            do_login(server=ASKANNA_API_SERVER)
    else:
        do_login(server=ASKANNA_API_SERVER)
